'use strict';

var isFiniteNumber = require('../../core/utils/is-finite-number.js');
var patternRegexpUtils = require('../utils/pattern-regexp-utils.js');
var multiplyByPowerOfTen = require('../../core/utils/multiply-by-power-of-ten.js');
var formattedStringToNumber = require('../parse/utils/formatted-string-to-number.js');
var numberToFormattedNumber = require('../format/number-to-formatted-number/number-to-formatted-number.js');

/**
 * Basis point format (BPS)
 * <i> See https://en.wikipedia.org/wiki/Basis_point
 */
const bpsFormatter = {
    name: 'bps',
    regexps: {
        format: /BPS/,
        unformat: /BPS/,
    },
    format: (number, pattern, options) => {
        const scaledValue = multiplyByPowerOfTen(number, 4);
        const patternWithEscapedBPS = patternRegexpUtils.patternReplace(pattern, /BPS/, `'ɵBPSɵ'`);
        const formatResult = numberToFormattedNumber(scaledValue, patternWithEscapedBPS, options);
        return formatResult.replace('ɵBPSɵ', 'BPS');
    },
    unformat: (string, options) => {
        const number = formattedStringToNumber(string.replace(/\s?BPS/, ''), options);
        return isFiniteNumber(number) ? multiplyByPowerOfTen(number, -4) : number;
    },
};

module.exports = bpsFormatter;
