'use strict';

var patternRegexpUtils = require('../utils/pattern-regexp-utils.js');
var numberToFormattedNumber = require('../format/number-to-formatted-number/number-to-formatted-number.js');

const currencySymbolsMap = {
    EUR: '€',
    USD: '$',
    XCD: 'EC$',
    AUD: 'A$',
    INR: '₹',
    BRL: 'R$',
    CAD: 'CA$',
    XAF: 'FCFA',
    CNY: 'CN¥',
    NZD: 'NZ$',
    XPF: 'CFPF',
    GBP: '£',
    HKD: 'HK$',
    ILS: '₪',
    JPY: '¥',
    KRW: '₩',
    XOF: 'CFA',
    MXN: 'MX$',
    TWD: 'NT$',
    VND: '₫',
};
const currencyFormatter = {
    name: 'currency',
    regexps: {
        format: /(\$)/,
    },
    format: (number, pattern, options) => {
        var _a;
        const currencyFromOptions = (_a = options.currency) === null || _a === void 0 ? void 0 : _a.toUpperCase();
        const localizedCurrencySymbol = currencySymbolsMap[currencyFromOptions] || currencyFromOptions || '';
        const patternWithEscapedCurrencySymbol = patternRegexpUtils.patternReplace(pattern, /\$/, `'ɵcurrencyɵ'`);
        const formatResult = numberToFormattedNumber(number, patternWithEscapedCurrencySymbol, options);
        return formatResult.replace('ɵcurrencyɵ', _ => localizedCurrencySymbol);
    },
};

module.exports = currencyFormatter;
