'use strict';

var isNanNumber = require('../../core/utils/is-nan-number.js');
var isFiniteNumber = require('../../core/utils/is-finite-number.js');
var patternRegexpUtils = require('../utils/pattern-regexp-utils.js');
var multiplyByPowerOfTen = require('../../core/utils/multiply-by-power-of-ten.js');
var formattedStringToNumber = require('../parse/utils/formatted-string-to-number.js');
var splitStringInTwoParts = require('../../core/utils/split-string-in-two-parts.js');
var numberToFormattedNumber = require('../format/number-to-formatted-number/number-to-formatted-number.js');

const exponentialFormatter = {
    name: 'exponential',
    regexps: {
        format: /[eE][+-][0-9]+/,
        unformat: /[eE][+-][0-9]+/,
    },
    format: (number, pattern, options) => {
        const exponential = typeof number === 'number' && !isNanNumber(number) ? number.toExponential() : '0e+0';
        const parts = splitStringInTwoParts(exponential, 'e');
        const patternWithoutExponential = patternRegexpUtils.patternReplace(pattern, /e[+|-]{1}0/i, '');
        const formatResult = numberToFormattedNumber(+parts[0], patternWithoutExponential, options);
        return formatResult + 'e' + parts[1];
    },
    unformat: (string, options) => {
        var _a;
        const value = formattedStringToNumber(string.replace(/e[+-]{1}[0-9]{1,3}/i, ''), options);
        const powerOfTenExponent = +(((_a = string.match(/e([+-]{1}[0-9]{1,3})/i)) === null || _a === void 0 ? void 0 : _a[1]) || '0');
        return isFiniteNumber(value) ? multiplyByPowerOfTen(value, powerOfTenExponent) : value;
    },
};

module.exports = exponentialFormatter;
