'use strict';

var patternRegexpUtils = require('../utils/pattern-regexp-utils.js');
var multiplyByPowerOfTen = require('../../core/utils/multiply-by-power-of-ten.js');
var formattedStringToNumber = require('../parse/utils/formatted-string-to-number.js');
var numberToFormattedNumber = require('../format/number-to-formatted-number/number-to-formatted-number.js');

const percentageFormatter = {
    name: 'percentage',
    regexps: {
        format: /%!?/,
        unformat: /%/,
    },
    format: (number, pattern, options) => {
        const hasNotScalePercentageSymbolInPattern = patternRegexpUtils.patternIncludes(pattern, '%!');
        const scaledValue = options.scalePercentage && !hasNotScalePercentageSymbolInPattern ? multiplyByPowerOfTen(number, 2) : number;
        const patternWithEscapedPercentage = patternRegexpUtils.patternReplace(pattern, /%!?/, `'ɵ%ɵ'`);
        const formatResult = numberToFormattedNumber(scaledValue, patternWithEscapedPercentage, options);
        return formatResult.replace('ɵ%ɵ', '%');
    },
    unformat: (string, options) => {
        const number = formattedStringToNumber(string.replace(/\s?%/, ''), options);
        return number && options.scalePercentage ? multiplyByPowerOfTen(number, -2) : number;
    },
};

module.exports = percentageFormatter;
