import isNil from '../../core/utils/is-nil.mjs';
import isFunction from '../../core/utils/is-function.mjs';
import isNaNNumber from '../../core/utils/is-nan-number.mjs';
import formattedStringToNumber from './utils/formatted-string-to-number.mjs';
import resolveFormatOptions from '../utils/resolve-format-options.mjs';

const getUnformatFunctionIfMatch = (input, resolvedOptions) => {
    for (const formatter of resolvedOptions.formatters) {
        const matcher = formatter.regexps.unformat;
        if (!matcher)
            continue;
        const matcherResult = isFunction(matcher) ? matcher(input, resolvedOptions) : !!input.match(matcher);
        if (matcherResult)
            return formatter.unformat;
    }
};
const parse = (input, options) => {
    const resolvedOptions = resolveFormatOptions(options);
    let value;
    if (isNil(input) || isNaNNumber(input)) {
        value = null;
    }
    else if (typeof input === 'number') {
        // Handles negative zero
        value = input === 0 ? 0 : input;
    }
    else if (typeof input === 'string') {
        if (resolvedOptions.zeroFormat && input === resolvedOptions.zeroFormat) {
            value = 0;
        }
        else if (resolvedOptions.nullFormat && input === resolvedOptions.nullFormat) {
            value = null;
        }
        else {
            const unformatFunctionFromFormatters = getUnformatFunctionIfMatch(input, resolvedOptions);
            const unformatFunction = unformatFunctionFromFormatters || formattedStringToNumber;
            value = unformatFunction(input, resolvedOptions);
        }
    }
    else {
        const result = +input;
        value = result === 0 ? result : (result || null);
    }
    return value;
};

export default parse;
